package com.timetec.blesdktestapp;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.GridLayout;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.timetec.bleandroiddevelopmentkit.TimetecBleSdk;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class BLEConnectionActivity extends AppCompatActivity {

    private static final int OPERATION_LEVEL_SELECT = 100;
    private static final int OPERATION_WRITE_KEYFOLD = 200;
    private static final int OPERATION_WRITE_CARD = 300;

    private static final int ADD = 1;
    private static final int DELETE = 2;

    Button btn1, btn2, btn3, btn4, btn5, btn6, btn7, btn8, btn9, btn10,
            btn11, btn12, btn13, btn14, btn15, btn16;

    Button btnAddData, btnDeleteData, btnClearData, btnOperationType;

    TimetecBleSdk timetecBleSdk; //Class from TimetecBleSdk
    BluetoothAdapter mBluetoothAdapter;
    BluetoothDevice bluetoothDevice;
    BluetoothLeScanner leScanner;
    BluetoothGatt mBluetoothGattConnected;
    BluetoothGattCharacteristic mBluetoothGattCharacteristic;
    private Handler mHandler = new Handler();
    List<ScanFilter> filters = new ArrayList<>();
    ScanSettings settings;
    boolean connecting;
    boolean attempReconnection = true;
    int currentOperation = OPERATION_LEVEL_SELECT;


    String macAddress;

    GridLayout buttonGrid;
    LinearLayout progressLayout, linearConnectedTo, linearOperationType, linearWriteData, linearAttemptingReconnect;
    TextView tvProgress, tvMacAddress, tvWriteType;
    AlertDialog alertDialog;

    private final static UUID BLE16SERVICEUUID = UUID.fromString(TimetecBleSdk.BLE_SERVICE_UUID);
    private final static UUID BLE16CHARACTERISTICUUID = UUID.fromString(TimetecBleSdk.BLE_CONTROL_CHARACTERISTIC_UUID);

    private BluetoothGattCallback mGattCallback = new BluetoothGattCallback() {

        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if (status == BluetoothGatt.GATT_SUCCESS && newState == BluetoothProfile.STATE_CONNECTED) {
                runOnUiThread(() -> linearAttemptingReconnect.setVisibility(View.GONE));
                gatt.discoverServices();
            } else if (status == BluetoothGatt.GATT_SUCCESS && newState == BluetoothProfile.STATE_DISCONNECTED) {
                //Don't reconnect already after finishing the activity
                if (attempReconnection) {
                    runOnUiThread(() -> linearAttemptingReconnect.setVisibility(View.VISIBLE));
                    gatt.connect();
                }
                Log.d("Gatt", "Disconnected");
            } else {
                runOnUiThread(() -> linearAttemptingReconnect.setVisibility(View.VISIBLE));
                Log.d("Gatt", "status:" + status + ", newState:" + newState);
            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                mBluetoothGattCharacteristic = gatt.getService(BLE16SERVICEUUID).getCharacteristic(BLE16CHARACTERISTICUUID);
                runOnUiThread(() -> dismissProgress());
                Log.d("MainActivity", "GAttConnected!");
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
        }
    };

    private ScanCallback scanCallback = new ScanCallback() {

        @Override
        public void onScanResult(int callbackType, ScanResult result) {
            if (result != null) {
                Log.d("Detected Device", result.getDevice().getAddress());
                if (result.getDevice().getAddress().equalsIgnoreCase(macAddress) && !connecting) {
                    bluetoothDevice = result.getDevice();
                    connecting = true;
                    connectDevice();
                }
            }
        }

        @Override
        public void onBatchScanResults(List<ScanResult> results) {
            super.onBatchScanResults(results);
            for (ScanResult r : results) {
                Log.d("Detected Device Batch", r.getDevice().getAddress());
            }

        }

        @Override
        public void onScanFailed(int errorCode) {
            super.onScanFailed(errorCode);
            Log.d("Detected Device", "Failed!");

        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bleconnection);

        findById();
        setOnClick();

        macAddress = getIntent().getStringExtra("macAddress");

        timetecBleSdk = new TimetecBleSdk(BLEConnectionActivity.this);
        try {
            if (!timetecBleSdk.setMacAddress(macAddress)) {
                Toast.makeText(BLEConnectionActivity.this,
                        "setMacAddress() received an invalid/unauthorized mac address! Activity stop soon!",
                        Toast.LENGTH_LONG).show();

                mHandler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        finish();
                    }
                }, 4500);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        BluetoothManager manager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        if (manager != null) {
            mBluetoothAdapter = manager.getAdapter();
            leScanner = mBluetoothAdapter.getBluetoothLeScanner();
            settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
            scanLeDevice(true);
        }
    }

    private void findById() {
        btn1 = findViewById(R.id.btn_1);
        btn2 = findViewById(R.id.btn_2);
        btn3 = findViewById(R.id.btn_3);
        btn4 = findViewById(R.id.btn_4);
        btn5 = findViewById(R.id.btn_5);
        btn6 = findViewById(R.id.btn_6);
        btn7 = findViewById(R.id.btn_7);
        btn8 = findViewById(R.id.btn_8);
        btn9 = findViewById(R.id.btn_9);
        btn10 = findViewById(R.id.btn_10);
        btn11 = findViewById(R.id.btn_11);
        btn12 = findViewById(R.id.btn_12);
        btn13 = findViewById(R.id.btn_13);
        btn14 = findViewById(R.id.btn_14);
        btn15 = findViewById(R.id.btn_15);
        btn16 = findViewById(R.id.btn_16);

        progressLayout = findViewById(R.id.linearProgress);
        buttonGrid = findViewById(R.id.buttonGrid);
        tvProgress = findViewById(R.id.tvProgress);

        linearConnectedTo = findViewById(R.id.linearConnectedTo);
        tvMacAddress = findViewById(R.id.tvMacAddress);
        tvWriteType = findViewById(R.id.tv_writeType);

        linearAttemptingReconnect = findViewById(R.id.linear_attempting_reconnect);

        linearOperationType = findViewById(R.id.linearOperationType);
        linearWriteData = findViewById(R.id.linearWriteData);
        btnAddData = findViewById(R.id.btn_addData);
        btnDeleteData = findViewById(R.id.btn_deleteData);
        btnClearData = findViewById(R.id.btn_clearData);

        btnOperationType = findViewById(R.id.btn_operationType);
    }

    private void setOnClick() {
        btn1.setOnClickListener(v -> sendDataToGatt(1));

        btn2.setOnClickListener(v -> sendDataToGatt(2));

        btn3.setOnClickListener(v -> sendDataToGatt(3));

        btn4.setOnClickListener(v -> sendDataToGatt(4));

        btn5.setOnClickListener(v -> sendDataToGatt(5));

        btn6.setOnClickListener(v -> sendDataToGatt(6));

        btn7.setOnClickListener(v -> sendDataToGatt(7));

        btn8.setOnClickListener(v -> sendDataToGatt(8));

        btn9.setOnClickListener(v -> sendDataToGatt(9));

        btn10.setOnClickListener(v -> sendDataToGatt(10));

        btn11.setOnClickListener(v -> sendDataToGatt(11));

        btn12.setOnClickListener(v -> sendDataToGatt(12));

        btn13.setOnClickListener(v -> sendDataToGatt(13));

        btn14.setOnClickListener(v -> sendDataToGatt(14));

        btn15.setOnClickListener(v -> sendDataToGatt(15));

        btn16.setOnClickListener(v -> sendDataToGatt(16));

        btnOperationType.setOnClickListener(v -> {
            if (currentOperation == OPERATION_LEVEL_SELECT) {
                currentOperation = OPERATION_WRITE_KEYFOLD;
            } else if (currentOperation == OPERATION_WRITE_KEYFOLD) {
                currentOperation = OPERATION_WRITE_CARD;
            } else if (currentOperation == OPERATION_WRITE_CARD) {
                currentOperation = OPERATION_LEVEL_SELECT;
            }

            displayUI();
        });

        btnAddData.setOnClickListener(v -> showOperationDialog(ADD));

        btnDeleteData.setOnClickListener(v -> showOperationDialog(DELETE));

        btnClearData.setOnClickListener(v -> sendDataToGattClear());
    }

    private void sendDataToGatt(int channel) {
        mBluetoothGattCharacteristic.setValue(timetecBleSdk.getParamBle16(channel));
        mBluetoothGattConnected.writeCharacteristic(mBluetoothGattCharacteristic);
    }

    private void sendDataToGatt(String data, int operation) {
        if (currentOperation == OPERATION_WRITE_CARD) {
            mBluetoothGattCharacteristic.setValue(timetecBleSdk.getParamCard(data, operation));
        } else if (currentOperation == OPERATION_WRITE_KEYFOLD) {
            Log.d("sendDataToGatt()", "operation write keyfold");
            mBluetoothGattCharacteristic.setValue(timetecBleSdk.getParamKeyfold(data, operation));
        }

        mBluetoothGattConnected.writeCharacteristic(mBluetoothGattCharacteristic);
    }

    private void sendDataToGattClear() {
        if (currentOperation == OPERATION_WRITE_CARD) {
            mBluetoothGattCharacteristic.setValue(timetecBleSdk.getParamCardClear());
        } else if (currentOperation == OPERATION_WRITE_KEYFOLD) {
            mBluetoothGattCharacteristic.setValue(timetecBleSdk.getParamKeyfoldClear());
        }
        mBluetoothGattConnected.writeCharacteristic(mBluetoothGattCharacteristic);
    }

    private void connectDevice() {
        if (bluetoothDevice != null) {
            showProgress();
            mBluetoothGattConnected = bluetoothDevice.connectGatt(this, true, mGattCallback);
            scanLeDevice(false);
        }
    }

    private void showProgress() {
        progressLayout.setVisibility(View.VISIBLE);
        tvProgress.setText(macAddress);
        buttonGrid.setVisibility(View.GONE);
        linearConnectedTo.setVisibility(View.GONE);
    }

    private void dismissProgress() {
        progressLayout.setVisibility(View.GONE);
        tvMacAddress.setText(macAddress);
        linearConnectedTo.setVisibility(View.VISIBLE);
        linearOperationType.setVisibility(View.VISIBLE);
        displayUI();
    }

    private void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            mHandler.postDelayed(() -> {
                Log.d("MainActivity", "Scan Stopped");
                leScanner.stopScan(scanCallback);
            }, 2500);

            Log.d("MainActivity", "Scan Started");
            leScanner.startScan(filters, settings, scanCallback);
        } else {
            leScanner.stopScan(scanCallback);
        }
    }

    private void displayUI() {
        if (currentOperation == OPERATION_LEVEL_SELECT) {
            btnOperationType.setText("Level Control");
            tvWriteType.setText("Level Control");
            buttonGrid.setVisibility(View.VISIBLE);
            linearWriteData.setVisibility(View.GONE);
        } else if (currentOperation == OPERATION_WRITE_KEYFOLD) {
            buttonGrid.setVisibility(View.GONE);
            btnOperationType.setText("Write Keyfold");
            tvWriteType.setText("Write Keyfold");
            linearWriteData.setVisibility(View.VISIBLE);
        } else if (currentOperation == OPERATION_WRITE_CARD) {
            buttonGrid.setVisibility(View.GONE);
            btnOperationType.setText("Write Card");
            tvWriteType.setText("Write Card");
            linearWriteData.setVisibility(View.VISIBLE);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        attempReconnection = false;
        if (mBluetoothGattConnected != null) {
            mBluetoothGattConnected.disconnect();
        }
    }

    private void showOperationDialog(final int operation) {
        LayoutInflater inflater = this.getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.write_dialog, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(BLEConnectionActivity.this);
        builder.setView(dialogView);


        final EditText editText = dialogView.findViewById(R.id.spinner_levels);
        final Button btnOk = dialogView.findViewById(R.id.btnOk);
        TextView promptText = dialogView.findViewById(R.id.promptText);
        Button btnCancel = dialogView.findViewById(R.id.btnCancel);

        if (currentOperation == OPERATION_WRITE_KEYFOLD) {
            promptText.setText("Enter your Keyfold ID");
            editText.setHint("Keyfold ID");
        } else if (currentOperation == OPERATION_WRITE_CARD) {
            promptText.setText("Enter your card ID");
            editText.setHint("Card Id");
        }

        btnCancel.setOnClickListener(v -> alertDialog.dismiss());

        btnOk.setOnClickListener(v -> {
            switch (currentOperation) {
                case OPERATION_WRITE_CARD:
                    if (operation == ADD) {
                        if (!editText.getText().toString().isEmpty()) {
                            sendDataToGatt(editText.getText().toString(), TimetecBleSdk.OPERATION_ADD);
                            alertDialog.dismiss();
                        } else {
                            editText.setError("Required field!");
                        }
                    } else if (operation == DELETE) {
                        sendDataToGatt(editText.getText().toString(), TimetecBleSdk.OPERATION_DELETE);
                        alertDialog.dismiss();
                    }
                    break;
                case OPERATION_WRITE_KEYFOLD:
                    if (operation == ADD) {
                        sendDataToGatt(editText.getText().toString().toUpperCase(), TimetecBleSdk.OPERATION_ADD);
                        alertDialog.dismiss();
                    } else if (operation == DELETE) {
                        sendDataToGatt(editText.getText().toString().toUpperCase(), TimetecBleSdk.OPERATION_DELETE);
                        alertDialog.dismiss();
                    }
                    break;
            }
        });
        builder.setCancelable(false);
        alertDialog = builder.show();
    }
}
