package com.timetec.blesdktestapp;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.timetec.bleandroiddevelopmentkit.TimetecBleKeyFold;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

//Similar to BLEConnectionActivity, but this is for writing data to keyfold.
public class KeyfoldConnectionActivity extends AppCompatActivity {
    public static final String TAG = "KeyfoldConnectionActivity";
    public static final int WRITE_FLOOR = 2;

    private final UUID KEYFOLDSERVICEUUID = UUID.fromString(TimetecBleKeyFold.KEYFOLD_SETTING_SERVICE_UUID);
    private final UUID KEYFOLDCHARACTERISTICUUID = UUID.fromString(TimetecBleKeyFold.KEYFOLD_SETTING_CHARACTERISTIC_UUID);

    private TimetecBleKeyFold bleKeyFold;
    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothDevice bluetoothDevice;
    private BluetoothLeScanner leScanner;
    private BluetoothGatt mBluetoothGattConnected;
    private BluetoothGattCharacteristic mBluetoothGattCharacteristic;
    private Handler mHandler = new Handler();
    private List<ScanFilter> filters = new ArrayList<>();
    private ScanSettings settings;
    private boolean connecting;
    private boolean attemptReconnect = true;
    private String macAddress;

    private LinearLayout linearProgress, linearConnectedTo, linearControlLayout, linearOperationType;
    private TextView tvProgress;
    private TextView tvMacAddress;
    //The button will display a different name, just UI changes, programatically, the buttons are still
    //recognize as such:
    // 3    4
    // 1    2
    //Only the name 3->"1", 4->"2", 1->"3", 2->"4"
    private Button btnWriteType, btn1, btn2, btn3, btn4;
    private AlertDialog alertDialog;

    private int currentOperation = WRITE_FLOOR;

    private BluetoothGattCallback mGattCallback = new BluetoothGattCallback() {

        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if (status == BluetoothGatt.GATT_SUCCESS && newState == BluetoothProfile.STATE_CONNECTED) {
                gatt.discoverServices();
            } else if (status == BluetoothGatt.GATT_SUCCESS && newState == BluetoothProfile.STATE_DISCONNECTED) {
                if (attemptReconnect) {
                    gatt.connect();
                }
                Log.d("Gatt", "Disconnected");
            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        linearProgress.setVisibility(View.GONE);
                        linearControlLayout.setVisibility(View.VISIBLE);
                        linearOperationType.setVisibility(View.VISIBLE);
                        tvMacAddress.setText(macAddress);
                        linearConnectedTo.setVisibility(View.VISIBLE);
                    }
                });
                mBluetoothGattCharacteristic = gatt.getService(KEYFOLDSERVICEUUID).getCharacteristic(KEYFOLDCHARACTERISTICUUID);
                if (mBluetoothGattCharacteristic == null) {
                    Log.d("mBluetoothGattCharacteristic", "null");
                }
                Log.d("MainActivity", "GAttConnected!");
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
        }
    };

    private ScanCallback scanCallback = new ScanCallback() {

        @Override
        public void onScanResult(int callbackType, ScanResult result) {
            if (result != null) {
                Log.d("Detected Device", result.getDevice().getAddress());
                if (result.getDevice().getAddress().equalsIgnoreCase(macAddress) && !connecting) {
                    bluetoothDevice = result.getDevice();
                    connecting = true;
                    connectDevice();
                }
            }
        }

        @Override
        public void onBatchScanResults(List<ScanResult> results) {
            super.onBatchScanResults(results);
            for (ScanResult r : results) {
                Log.d("Detected Device Batch", r.getDevice().getAddress());
            }

        }

        @Override
        public void onScanFailed(int errorCode) {
            super.onScanFailed(errorCode);
            Log.d("Detected Device", "Failed!");

        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_keyfold_connection);

        findById();
        setOnClick();

        bleKeyFold = new TimetecBleKeyFold();
        macAddress = getIntent().getStringExtra("macAddress");

        BluetoothManager manager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        if (manager != null) {
            mBluetoothAdapter = manager.getAdapter();
            leScanner = mBluetoothAdapter.getBluetoothLeScanner();
            settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
            scanLeDevice(true);
        }
    }

    private void findById() {
        linearProgress = findViewById(R.id.linearProgress);
        linearConnectedTo = findViewById(R.id.linearConnectedTo);
        linearControlLayout = findViewById(R.id.linearControlLayout);
        linearOperationType = findViewById(R.id.linearOperationType);
        tvProgress = findViewById(R.id.tvProgress);
        tvMacAddress = findViewById(R.id.tvMacAddress);
        btnWriteType = findViewById(R.id.btn_writeType);
        btn1 = findViewById(R.id.btn_1);
        btn2 = findViewById(R.id.btn_2);
        btn3 = findViewById(R.id.btn_3);
        btn4 = findViewById(R.id.btn_4);
    }

    private void setOnClick() {
        btnWriteType.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        btn1.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showOperationDialog(1);
            }
        });

        btn2.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showOperationDialog(2);

            }
        });

        btn3.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showOperationDialog(3);
            }
        });

        btn4.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showOperationDialog(4);
            }
        });
    }

    private void sendDataToGatt(int buttonNo, String data) {
        Log.d(TAG, data);
        if (currentOperation == WRITE_FLOOR && TextUtils.isDigitsOnly(data)) {
            mBluetoothGattCharacteristic.setValue(bleKeyFold.getSetFloorParam(buttonNo, Integer.parseInt(data)));
        }
        mBluetoothGattConnected.writeCharacteristic(mBluetoothGattCharacteristic);
    }

    private void connectDevice() {
        if (bluetoothDevice != null) {
            linearProgress.setVisibility(View.VISIBLE);
            tvProgress.setText(macAddress);
            linearControlLayout.setVisibility(View.GONE);
            linearOperationType.setVisibility(View.GONE);
            linearConnectedTo.setVisibility(View.GONE);
            mBluetoothGattConnected = bluetoothDevice.connectGatt(this, true, mGattCallback);
            scanLeDevice(false);
        }
    }

    private void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    Log.d("MainActivity", "Scan Stopped");
                    leScanner.stopScan(scanCallback);
                }
            }, 2500);

            Log.d("MainActivity", "Scan Started");
            leScanner.startScan(filters, settings, scanCallback);
        } else {
            leScanner.stopScan(scanCallback);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        attemptReconnect = false;
        if (mBluetoothGattConnected != null) {
            mBluetoothGattConnected.disconnect();
        }
    }

    private void showOperationDialog(final int buttonNo) {
        String[] levelsArray = new String[]{"1", "2", "3", "4", "5", "6", "7", "8", "9", "10",
                "11", "12", "13", "14", "15", "16"};

        LayoutInflater inflater = this.getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.keyfold_write_dialog, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(KeyfoldConnectionActivity.this);
        builder.setView(dialogView);


        final Spinner spinner = dialogView.findViewById(R.id.spinner_levels);
        final Button btnOk = dialogView.findViewById(R.id.btnOk);
        TextView promptText = dialogView.findViewById(R.id.promptText);
        Button btnCancel = dialogView.findViewById(R.id.btnCancel);

        ArrayAdapter<String> adapter = new ArrayAdapter<>(
                this, android.R.layout.simple_spinner_item, Arrays.asList(levelsArray));
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);

        promptText.setText("Enter your level for this button");

        btnCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                alertDialog.dismiss();
            }
        });

        btnOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendDataToGatt(buttonNo, spinner.getSelectedItem().toString());
                alertDialog.dismiss();
            }
        });
        builder.setCancelable(false);
        alertDialog = builder.show();
    }
}
