package com.timetec.blesdktestapp;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.timetec.bleandroiddevelopmentkit.TimetecBleSdk;
import com.timetec.bleandroiddevelopmentkit.TimetecBleSdkAuth;
import com.timetec.bleandroiddevelopmentkit.TimetecBleSdkEventListener;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = MainActivity.class.getSimpleName();
    private static final int REQUEST_ENABLE_BT = 1;
    private static final int REQUEST_ENABLE_GPS = 2;


    //TimetecBleSdk Auth for license verification
    TimetecBleSdkAuth timetecBleSdkAuth = new TimetecBleSdkAuth(MainActivity.this);
    TimetecBleSdk timetecBleSdk;
    BluetoothAdapter mBluetoothAdapter;
    BluetoothLeScanner leScanner;
    ScanSettings settings;
    ArrayList<BluetoothDevice> mDevices = new ArrayList<>();
    ArrayList<String> listAddresses = new ArrayList<>(); //Must tally with mDevices, just exist to prevent existing scanned devices
    List<ScanFilter> filters = new ArrayList<>();
    ListView listDevices;
    DeviceAdapter deviceAdapter;

    TextView textViewVerify;
    EditText edtLicenseId;
    Button btnVerify, btnRescan;
    ProgressBar progressBarVerify, progressBarConnect;

    //Handler to run background threads
    private Handler mHandler = new Handler();

    //Call backs for Bluetooth scanning
    private ScanCallback scanCallback = new ScanCallback() {

        @Override
        public void onScanResult(int callbackType, ScanResult result) {
            if (result != null) {
                String name = result.getDevice().getName();
                if (!listAddresses.contains(result.getDevice().getAddress()) && (name != null && !name.isEmpty())) {
                    listAddresses.add(result.getDevice().getAddress());
                    mDevices.add(result.getDevice());
                    deviceAdapter.notifyDataSetChanged();
                }
                Log.d("Detected Device", result.getDevice().getAddress());
            }
        }

        @Override
        public void onBatchScanResults(List<ScanResult> results) {
            super.onBatchScanResults(results);
            for (ScanResult r : results) {
                Log.d("Detected Device Batch", r.getDevice().getAddress());
            }

        }

        @Override
        public void onScanFailed(int errorCode) {
            super.onScanFailed(errorCode);
            Log.d("Detected Device", "Failed!");

        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        findbyId();
        setOnClick();

        //Checks for Bluetooth and GPS permission
        //GPS permission is required for bluetooth scanning to work, as told in the official android SDK documentation
        if (ContextCompat.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH) != PackageManager.PERMISSION_GRANTED
                && ContextCompat.checkSelfPermission(MainActivity.this, Manifest.permission.BLUETOOTH_ADMIN) != PackageManager.PERMISSION_GRANTED) {

            // Permission is not granted
            if (ActivityCompat.shouldShowRequestPermissionRationale(MainActivity.this, Manifest.permission.BLUETOOTH)
                    || ActivityCompat.shouldShowRequestPermissionRationale(MainActivity.this, Manifest.permission.BLUETOOTH_ADMIN)) {
                Toast.makeText(MainActivity.this, "Bluetooth Required!", Toast.LENGTH_SHORT).show();
            } else {
                // No explanation needed; request the permission
                ActivityCompat.requestPermissions(MainActivity.this,
                        new String[]{Manifest.permission.BLUETOOTH, Manifest.permission.BLUETOOTH_ADMIN, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION},
                        1);
            }
        } else {
            mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
            LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

            if (mBluetoothAdapter.isEnabled()) {
//                boolean gps_enabled = false;
//                if (locationManager != null) {
//                    gps_enabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
//
//                }

//                if(gps_enabled) {
                leScanner = mBluetoothAdapter.getBluetoothLeScanner();
                settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
                preScan();
//                } else {
//                    showDialogBox(REQUEST_ENABLE_GPS);
//                }
            } else {
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
            }
        }

    }

    //Function for starting a bluetooth scan
    private void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    Log.d("MainActivity", "Scan Stopped");
                    leScanner.stopScan(scanCallback);
                }
            }, 2500);

            Log.d("MainActivity", "Scan Started");
            leScanner.startScan(filters, settings, scanCallback);
        } else {
            leScanner.stopScan(scanCallback);
        }
    }


    //Find UI components
    private void findbyId() {

        progressBarVerify = findViewById(R.id.progressBarVerify);
        textViewVerify = findViewById(R.id.tvVerify);

        progressBarConnect = findViewById(R.id.progressBarConnect);

        edtLicenseId = findViewById(R.id.edtLicenseId);

        btnVerify = findViewById(R.id.btnVerify);
        btnRescan = findViewById(R.id.btnRescan);

        btnVerify.performClick();

        listDevices = findViewById(R.id.listDevices);
        deviceAdapter = new DeviceAdapter(this, 0, mDevices);
        listDevices.setAdapter(deviceAdapter);
    }

    //Set UI component actions
    private void setOnClick() {
        listDevices.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                //Quintic or TTC are Names for Keyfolds
                if (mDevices.get(position).getName().contains("TTC") || mDevices.get(position).getName().contains("Quintic")) {
                    Intent intent = new Intent(MainActivity.this, KeyfoldConnectionActivity.class);
                    intent.putExtra("macAddress", mDevices.get(position).getAddress());
                    startActivity(intent);
                } else {
                    try {
                        //Set Mac Address to check the mac address is authorized under license
                        if (timetecBleSdk.setMacAddress(mDevices.get(position).getAddress())) {
                            Intent intent = new Intent(MainActivity.this, BLEConnectionActivity.class);
                            intent.putExtra("macAddress", mDevices.get(position).getAddress());
                            startActivity(intent);
                        } else {
                            Toast.makeText(MainActivity.this, "Unauthorized Mac Address!", Toast.LENGTH_LONG).show();
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        });

        btnRescan.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                listAddresses.clear();
                mDevices.clear();
                deviceAdapter.notifyDataSetChanged();
                scanLeDevice(true);
            }
        });

        btnVerify.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String licenseId = edtLicenseId.getText().toString();

                try {
                    //To authenticate license id in the edittext
                    //NOTE: KEEP YOUR LICENSE ID A SECRET!s
                    timetecBleSdkAuth.verifyLicense(licenseId, new TimetecBleSdkEventListener() {
                        @Override
                        public void onGetMacAddress() {
                            Log.d(TAG, "onGetMacAddress Called");
                            progressBarVerify.setVisibility(View.VISIBLE);
                            textViewVerify.setText("Verifying");
                            textViewVerify.setVisibility(View.VISIBLE);
                        }

                        @Override
                        public void onGetMacAddressSuccess() {
                            Log.d(TAG, "onGetMacAddress success Called");
                            progressBarVerify.setVisibility(View.GONE);
                            textViewVerify.setTextColor(getResources().getColor(R.color.Green));
                            textViewVerify.setText("Verification Success!");
                            textViewVerify.setVisibility(View.VISIBLE);
                        }

                        @Override
                        public void onGetMacAddressFailed() {
                            Log.d(TAG, "onGetMacAddress failed Called");
                            progressBarVerify.setVisibility(View.GONE);
                            textViewVerify.setTextColor(getResources().getColor(R.color.Red));
                            textViewVerify.setText("Verification Failed!");
                            textViewVerify.setVisibility(View.VISIBLE);
                        }
                    });
                    if (timetecBleSdk == null) {
                        //getMacAddress Must be called at least once before an instance of TimetecBleSdk can be created!
                        //**At least once as in since the application is first installed.
                        timetecBleSdk = new TimetecBleSdk(MainActivity.this);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {

        switch (requestCode) {
            case 1: {
                if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    if (mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled()) {
                        Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                        startActivity(enableBtIntent);
                        finish();
                    } else {
                        scanLeDevice(true);
                    }

                    if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
                        finish();
                    }
                } else {
                    final AlertDialog.Builder builder = new AlertDialog.Builder(this);
                    builder.setTitle("Functionality limited");
                    builder.setMessage("Since location access has not been granted, this app will not be able to discover beacons when in the background.");
                    builder.setPositiveButton(android.R.string.ok, null);
                    builder.setOnDismissListener(new DialogInterface.OnDismissListener() {
                        @Override
                        public void onDismiss(DialogInterface dialog) {
                        }
                    });
                    builder.show();
                }
            }
        }
    }

    //Adapter to display the listview
    class DeviceAdapter extends ArrayAdapter<BluetoothDevice> {

        DeviceAdapter(@NonNull Context context, int resource, @NonNull List<BluetoothDevice> objects) {
            super(context, 0, objects);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            // Get the data item for this position
            BluetoothDevice device = getItem(position);
            // Check if an existing view is being reused, otherwise inflate the view
            if (convertView == null) {
                convertView = LayoutInflater.from(getContext()).inflate(android.R.layout.simple_list_item_2, parent, false);
            }
            // Lookup view for data population
            TextView tvName = convertView.findViewById(android.R.id.text1);
            TextView tvAddress = convertView.findViewById(android.R.id.text2);
            // Populate the data into the template view using the data object
            if (device != null) {
                tvName.setText(device.getName());
            }
            if (device != null) {
                tvAddress.setText(device.getAddress());
            }

            // Return the completed view to render on screen
            return convertView;
        }
    }

    //Make sure everything is ok befor start scanning
    private void preScan() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (this.checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                final AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("This app needs location access");
                builder.setMessage("Please grant location access so this app can detect beacons.");
                builder.setPositiveButton(android.R.string.ok, null);
                builder.setOnDismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialog) {
                        requestPermissions(new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, 1);
                    }
                });
                builder.show();
            } else {
                if (mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled()) {
                    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                    startActivity(enableBtIntent);
                    finish();
                } else {
                    scanLeDevice(true);
                }

                if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
                    finish();
                }
            }
        }
    }

    private void showDialogBox(final int request) {
        AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
        switch (request) {
            case REQUEST_ENABLE_GPS:
                builder.setMessage("Location Service disabled, Enable location service now?");
        }

        builder.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (request) {
                    case REQUEST_ENABLE_GPS:
                        Intent enableGPSIntent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                        startActivityForResult(enableGPSIntent, REQUEST_ENABLE_GPS);
                        break;
                }
            }
        });

        builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                finish();
            }
        });
        builder.show();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case REQUEST_ENABLE_BT:
                if (resultCode == RESULT_OK) {
//                    LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
//
//                    boolean gps_enabled = false;
//                    if (locationManager != null) {
//                        gps_enabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
//                    }
//
//                    if(gps_enabled) {
                    leScanner = mBluetoothAdapter.getBluetoothLeScanner();
                    settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
                    preScan();
//                    } else {
//                        showDialogBox(REQUEST_ENABLE_GPS);
//                    }
                } else {
                    finish();
                }
                break;
            case REQUEST_ENABLE_GPS:
                LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

                boolean gps_enabled = false;
                if (locationManager != null) {
                    gps_enabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
                }

                if (gps_enabled) {
                    leScanner = mBluetoothAdapter.getBluetoothLeScanner();
                    settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
                    preScan();
                } else {
                    finish();
                }
                break;
        }
    }
}
